import React, {useRef} from 'react';
import cn from 'classnames';

import styles from './SkillDropdownComponent.module.scss';
import DropArrow from '@assets/icons/ic_down-arrow.svg';
import EmptyIcon from '@assets/icons/ic_empty.svg';
import { resource } from '@assets/text/resources';
import IconSpinner from '@assets/icons/ic_spinner.svg';
import { Skill } from '@api/types';
import {useOnClickOutside} from "@src/hooks/useClickOutside";

type SkillDropdownComponentProps = {
    /** Show drop down flag */
    isShow: boolean;
    /** Available array of skills */
    skills: Array<Skill>;
    /** Selected skill id for current transaction */
    selectedSkillId?: string | null;
    /** Event handler for skill changes */
    handleSkillChange: (event: React.MouseEvent) => void;
    /** Event handler for hide drop down */
    hideDropdown: (event: MouseEvent | TouchEvent) => void;
    /** Event handler for show toggle */
    toggleShow: () => void;
    /** Is loading flag */
    isLoading: boolean;
    /** Disable flag for drop down */
    disabled: boolean;
};

/**
 *  Component to render a dropdown for display and select skills
 */
const SkillDropdownComponent = ({
    isShow,
    skills,
    selectedSkillId,
    handleSkillChange,
    hideDropdown,
    toggleShow,
    isLoading,
    disabled,
}: SkillDropdownComponentProps) => {
    const skillListItem = (skill: Skill) => (
        <button
            key={skill.id + skill.name}
            data-value={skill.id}
            onClick={handleSkillChange}
            type="button"
            title={skill.name}
        >
            {skill.name}
        </button>
    );

    const skillList =
        skills && skills.length > 0 ? (
            skills.map((skill) => skillListItem(skill))
        ) : (
            <div className={styles.empty}>
                <EmptyIcon className={styles.emptyIcon} />
                <span>{resource.title.noSkills}</span>
            </div>
        );

    const loadingWrapper = isLoading ? (
        <div className={styles.loadingContainer}>
            <IconSpinner />
            <span>{resource.title.skillsIsLoading}</span>
        </div>
    ) : (
        skillList
    );
    if (disabled) {
        return (
            <div className={styles.dropdown}>
                <button className={styles.dropButton} type="button">
                    <span className={styles.dropTitle}>
                        <span>
                            {selectedSkillId
                                ? skills.find((skill) => skill.id === selectedSkillId)?.name
                                : resource.button.selectSkill}
                        </span>
                    </span>
                </button>
            </div>
        );
    }

    const ref = useRef(null);

    useOnClickOutside(ref, hideDropdown)

    return (
        <div className={styles.dropdown} ref={ref}>
            <button
                className={cn([styles.dropButton, styles.dropButtonActive])}
                onClick={toggleShow}
                type="button"
            >
                <span className={styles.dropTitle}>
                    <span>
                        {selectedSkillId
                            ? skills.find((skill) => skill.id === selectedSkillId)?.name
                            : resource.button.selectSkill}
                    </span>
                    <DropArrow className={styles.dropArrow} />
                </span>
            </button>
            {isShow && <div className={styles.dropdownContent}>{loadingWrapper}</div>}
        </div>
    );
};

export default SkillDropdownComponent;
