import { initialState, reducer } from './reducer';
import MockFile from '../../utils/mockFileUtil';
import { Type } from './actionTypes';
import { INObjects } from '../types';
import { Transaction, TransactionStatus } from '../../api/types';

const testFile = MockFile({
    type: 'image/png',
    size: 50000,
});

describe('Transaction reducer tests', () => {
    it('Create local transaction success', () => {
        const action = {
            type: Type.CREATE_LOCAL_TRANSACTION_SUCCESS,
            payload: {
                transaction: {
                    uuid: 'uuid-1',
                    status: TransactionStatus.LOCAL,
                    transactionFile: testFile,
                },
            },
        };
        expect(reducer(initialState, action)).toEqual({
            ...initialState,
            byId: {
                ...initialState.byId,
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.LOCAL,
                    transactionFile: testFile,
                },
            },
            allIds: initialState.allIds.concat(['uuid-1']),
        });
    });

    it('Create transaction start', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.LOCAL,
                    transactionFile: testFile,
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.CREATE_TRANSACTION_START,
            payload: {
                uuid: 'uuid-1',
                skillId: 'skill-id',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                    status: 'New',
                    skillId: 'skill-id',
                },
            },
        });
    });

    it('Create transaction success', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.UPLOAD,
                    transactionFile: testFile,
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.CREATE_TRANSACTION_SUCCESS,
            payload: {
                uuid: 'uuid-1',
                transactionId: 'transaction-id',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                    status: 'New',
                    transactionId: 'transaction-id',
                },
            },
        });
    });

    it('Create transaction failed', () => {
        const initState: INObjects<Transaction> = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.UPLOAD,
                    transactionFile: testFile,
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.CREATE_TRANSACTION_ERROR,
            payload: {
                uuid: 'uuid-1',
                errorText: 'error text',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                    status: TransactionStatus.FAILED,
                },
            },
        });
    });

    it('Get transaction success', () => {
        const initState: INObjects<Transaction> = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSING,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.GET_TRANSACTION_SUCCESS,
            payload: {
                uuid: 'uuid-1',
                status: TransactionStatus.PROCESSED,
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                    status: TransactionStatus.PROCESSED,
                },
            },
        });
    });

    it('Download transaction file start', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSED,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.DOWNLOAD_TRANSACTION_FILE_START,
            payload: {
                uuid: 'uuid-1',
                fileId: 'file-id-1',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                },
            },
        });
    });

    it('Download transaction file success', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSED,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.DOWNLOAD_TRANSACTION_FILE_SUCCESS,
            payload: {
                uuid: 'uuid-1',
                fileId: 'file-id-1',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                },
            },
        });
    });

    it('Download transaction file error', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSED,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.DOWNLOAD_TRANSACTION_FILE_ERROR,
            payload: {
                uuid: 'uuid-1',
                errorText: 'error text',
                fileId: 'file-id-1',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                },
            },
        });
    });

    it('delete transaction success with one transaction in store', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSING,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.DELETE_TRANSACTION_SUCCESS,
            payload: {
                uuid: 'uuid-1',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {},
            allIds: [],
        });
    });

    it('delete transaction success with two transaction in store', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSING,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
                'uuid-2': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSING,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1', 'uuid-2'],
        };
        const action = {
            type: Type.DELETE_TRANSACTION_SUCCESS,
            payload: {
                uuid: 'uuid-1',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                'uuid-2': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSING,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-2'],
        });
    });

    it('Delete transaction file start', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSED,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.DELETE_TRANSACTION_START,
            payload: {
                uuid: 'uuid-1',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                    deleting: true,
                },
            },
        });
    });

    it('Delete transaction file error', () => {
        const initState = {
            byId: {
                'uuid-1': {
                    uuid: 'uuid-1',
                    status: TransactionStatus.PROCESSED,
                    transactionFile: testFile,
                    transactionId: 'transaction-id',
                },
            },
            allIds: ['uuid-1'],
        };
        const action = {
            type: Type.DELETE_TRANSACTION_ERROR,
            payload: {
                uuid: 'uuid-1',
                errorText: 'error text',
            },
        };
        expect(reducer(initState, action)).toEqual({
            ...initState,
            byId: {
                ...initState.byId,
                'uuid-1': {
                    ...initState.byId['uuid-1'],
                    deleting: false,
                },
            },
        });
    });
});
