import { runSaga } from 'redux-saga';
import transactionService, { GetTransactionInfoResponse } from '../../../api/transactionService';
import getTransactionSaga from './getTransactionSaga';
import { getTransactionError, getTransactionStart, getTransactionSuccess, TransactionActionTypes } from '../actions';
import { Type } from '../actionTypes';
import { resource } from '../../../assets/text/resources';
import { FileType, TransactionStatus } from '../../../api/types';

describe('Get transaction status sagas tests', () => {
    jest.spyOn(console, 'error').mockImplementation();
    it('Get transaction status success - Review link', async () => {
        const dispatched: Array<TransactionActionTypes> = [];
        const dummyData: GetTransactionInfoResponse = {
            data: {
                manualReviewLink: 'link',
                status: TransactionStatus.PROCESSING,
            },
            status: 200,
            statusText: 'OK',
            headers: {},
            config: {},
        };
        const getTransaction = jest
            .spyOn(transactionService, 'getTransactionInfo')
            .mockImplementation(() => Promise.resolve(dummyData));

        await runSaga(
            {
                dispatch: (action: TransactionActionTypes) => dispatched.push(action),
            },
            getTransactionSaga,
            {
                type: Type.GET_TRANSACTION_REQUEST,
                payload: {
                    uuid: 'uuid-1',
                    transactionId: 'transaction-id-1',
                    skillId: 'skill-1',
                },
            },
        ).toPromise();

        expect(dispatched).toContainEqual(
            getTransactionStart({
                uuid: 'uuid-1',
                transactionId: 'transaction-id-1',
                skillId: 'skill-1',
            }),
        );
        expect(dispatched).toContainEqual(
            getTransactionSuccess({
                uuid: 'uuid-1',
                status: TransactionStatus.REVIEW,
                reviewLink: 'link',
            }),
        );
        expect(getTransaction).toHaveBeenCalled();
    });

    it('Get transaction status success - Result Class', async () => {
        const dispatched: Array<TransactionActionTypes> = [];
        const dummyData: GetTransactionInfoResponse = {
            data: {
                documents: [
                    {
                        classification: {
                            resultClass: {
                                id: 'class-id-2',
                                name: 'class-2',
                            },
                        },
                    },
                ],
                status: TransactionStatus.PROCESSED,
            },
            status: 200,
            statusText: 'OK',
            headers: {},
            config: {},
        };
        const getTransaction = jest
            .spyOn(transactionService, 'getTransactionInfo')
            .mockImplementation(() => Promise.resolve(dummyData));

        await runSaga(
            {
                dispatch: (action: TransactionActionTypes) => dispatched.push(action),
            },
            getTransactionSaga,
            {
                type: Type.GET_TRANSACTION_REQUEST,
                payload: {
                    uuid: 'uuid-1',
                    transactionId: 'transaction-id-1',
                    skillId: 'skill-1',
                },
            },
        ).toPromise();

        expect(dispatched).toContainEqual(
            getTransactionStart({
                uuid: 'uuid-1',
                transactionId: 'transaction-id-1',
                skillId: 'skill-1',
            }),
        );
        expect(dispatched).toContainEqual(
            getTransactionSuccess({
                uuid: 'uuid-1',
                status: TransactionStatus.PROCESSED,
                resultClassName: 'class-2',
            }),
        );
        expect(getTransaction).toHaveBeenCalled();
    });

    it('Get transaction status success - Result file', async () => {
        const dispatched: Array<TransactionActionTypes> = [];
        const dummyData: GetTransactionInfoResponse = {
            data: {
                documents: [
                    {
                        resultFiles: [
                            {
                                fileId: 'file-id',
                                type: FileType.JSON,
                                fileName: 'file-name',
                            },
                        ],
                    },
                ],
                status: TransactionStatus.PROCESSED,
            },
            status: 200,
            statusText: 'OK',
            headers: {},
            config: {},
        };
        const getTransaction = jest
            .spyOn(transactionService, 'getTransactionInfo')
            .mockImplementation(() => Promise.resolve(dummyData));

        await runSaga(
            {
                dispatch: (action: TransactionActionTypes) => dispatched.push(action),
            },
            getTransactionSaga,
            {
                type: Type.GET_TRANSACTION_REQUEST,
                payload: {
                    uuid: 'uuid-1',
                    transactionId: 'transaction-id-1',
                    skillId: 'skill-1',
                },
            },
        ).toPromise();

        expect(dispatched).toContainEqual(
            getTransactionStart({
                uuid: 'uuid-1',
                transactionId: 'transaction-id-1',
                skillId: 'skill-1',
            }),
        );
        expect(dispatched).toContainEqual(
            getTransactionSuccess({
                uuid: 'uuid-1',
                status: TransactionStatus.PROCESSED,
                resultFiles: [{
                    fileId:'file-id',
                    fileName: 'file-name',
                    type: FileType.JSON
                }]
            }),
        );
        expect(getTransaction).toHaveBeenCalled();
    });

    it('Get transaction status success - Processing', async () => {
        const dispatched: Array<TransactionActionTypes> = [];
        const dummyData: GetTransactionInfoResponse = {
            data: {
                status: TransactionStatus.PROCESSING,
            },
            status: 200,
            statusText: 'OK',
            headers: {},
            config: {},
        };
        const getTransaction = jest
            .spyOn(transactionService, 'getTransactionInfo')
            .mockImplementation(() => Promise.resolve(dummyData));

        await runSaga(
            {
                dispatch: (action: TransactionActionTypes) => dispatched.push(action),
            },
            getTransactionSaga,
            {
                type: Type.GET_TRANSACTION_REQUEST,
                payload: {
                    uuid: 'uuid-1',
                    transactionId: 'transaction-id-1',
                    skillId: 'skill-1',
                },
            },
        ).toPromise();

        expect(dispatched).toContainEqual(
            getTransactionStart({
                uuid: 'uuid-1',
                transactionId: 'transaction-id-1',
                skillId: 'skill-1',
            }),
        );
        expect(dispatched).toContainEqual(
            getTransactionSuccess({
                uuid: 'uuid-1',
                status: TransactionStatus.PROCESSING,
            }),
        );
        expect(getTransaction).toHaveBeenCalled();
    });

    it('Get transaction status success - Failed', async () => {
        const dispatched: Array<TransactionActionTypes> = [];
        const dummyData: GetTransactionInfoResponse = {
            data: {
                status: TransactionStatus.FAILED,
            },
            status: 200,
            statusText: 'OK',
            headers: {},
            config: {},
        };
        const getTransaction = jest
            .spyOn(transactionService, 'getTransactionInfo')
            .mockImplementation(() => Promise.resolve(dummyData));

        await runSaga(
            {
                dispatch: (action: TransactionActionTypes) => dispatched.push(action),
            },
            getTransactionSaga,
            {
                type: Type.GET_TRANSACTION_REQUEST,
                payload: {
                    uuid: 'uuid-1',
                    transactionId: 'transaction-id-1',
                    skillId: 'skill-1',
                },
            },
        ).toPromise();

        expect(dispatched).toContainEqual(
            getTransactionStart({
                uuid: 'uuid-1',
                transactionId: 'transaction-id-1',
                skillId: 'skill-1',
            }),
        );
        expect(dispatched).toContainEqual(
            getTransactionSuccess({
                uuid: 'uuid-1',
                status: TransactionStatus.FAILED,
            }),
        );
        expect(getTransaction).toHaveBeenCalled();
    });

    it('Get transaction status failed', async () => {
        const dispatched: Array<TransactionActionTypes> = [];
        const getTransaction = jest
            .spyOn(transactionService, 'getTransactionInfo')
            .mockImplementation(() => Promise.reject());

        await runSaga(
            {
                dispatch: (action: TransactionActionTypes) => dispatched.push(action),
            },
            getTransactionSaga,
            {
                type: Type.GET_TRANSACTION_REQUEST,
                payload: {
                    uuid: 'uuid-1',
                    transactionId: 'transaction-id-1',
                    skillId: 'skill-1',
                },
            },
        ).toPromise();

        expect(dispatched).toContainEqual(
            getTransactionStart({
                uuid: 'uuid-1',
                transactionId: 'transaction-id-1',
                skillId: 'skill-1',
            }),
        );
        expect(dispatched).toContainEqual(
            getTransactionError({
                errorText: resource.error.getTransactionFailed,
            }),
        );
        expect(getTransaction).toHaveBeenCalled();
    });
});
