import { call, put, race, take } from 'redux-saga/effects';
import transactionService, { GetTransactionInfoResponse } from '../../../api/transactionService';
import {
    DeleteTransactionActionTypes,
    getTransactionError,
    getTransactionRequest,
    getTransactionStart,
    getTransactionSuccess,
} from '../actions';
import { Type } from '../actionTypes';
import { resource } from '../../../assets/text/resources';
import { ResultFile, TransactionStatus } from '../../../api/types';
import { SagaIterator } from 'redux-saga';

export default function* getTransactionSaga({ payload }: ReturnType<typeof getTransactionRequest>): SagaIterator {
    try {
        yield put(getTransactionStart(payload));
        const raceResult: { res: any; cancel: any } = yield race({
            res: call(transactionService.getTransactionInfo, payload.transactionId),
            cancel: take(
                (action: DeleteTransactionActionTypes) =>
                    action.type === Type.DELETE_TRANSACTION_SUCCESS && action.payload.uuid === payload.uuid,
            ),
        });
        if (raceResult.cancel) {
            return;
        }
        const response: GetTransactionInfoResponse = raceResult.res;

        const { data } = response;
        const { status } = response.data;

        let result: {
            uuid: string;
            status: TransactionStatus;
            fileName?: string;
            reviewLink?: string;
            resultFiles?: ResultFile[];
            resultClassName?: string;
        };

        if (data.manualReviewLink) {
            // If we found review link in response
            result = {
                uuid: payload.uuid,
                status: TransactionStatus.REVIEW,
                reviewLink: data.manualReviewLink,
            };
        } else {
            result = {
                uuid: payload.uuid,
                status,
            };

            // If we found file for download in response
            if (data.documents?.[0]?.resultFiles && data.documents?.[0]?.resultFiles.length > 0) {
                result.resultFiles = [];
                data.documents.forEach((document, index)=>{
                    let resultFiles: ResultFile[] = [];
                    if(document.resultFiles) {
                        resultFiles = document.resultFiles.map(file => {
                            file.fileIndex = index
                            return file
                        })
                    }
                    result.resultFiles = result.resultFiles?.concat(resultFiles)
                })
            }

            // If we found result class in response
            if (data.documents?.[0]?.classification?.resultClass) {
                const { resultClass } = data.documents?.[0]?.classification;
                result.resultClassName = resultClass.name ? resultClass.name : 'Unknown';
            }
        }

        if (data.sourceFiles && data.sourceFiles.length > 0 && data.sourceFiles[0].name) {
            result.fileName = data.sourceFiles[0].name;
        }

        yield put(getTransactionSuccess(result));
    } catch (error) {
        console.error(resource.error.getTransactionFailed);
        console.error(error);
        yield put(getTransactionError({ errorText: resource.error.getTransactionFailed }));
    }
}
