import React, {useCallback, useEffect, useState} from 'react';
import styles from './CardComponent.module.scss';
import StatusIconComponent from '@commonComponents/StatusIconComponent/StatusIconComponent';
import StatusTextComponent from '@commonComponents/StatusTextComponent/StatusTextComponent';
import PreviewComponent from '../PreviewComponent/PreviewComponent';
import ReviewResultComponent from '@commonComponents/ReviewResultComponent/ReviewResultComponent';
import SkillDropdownContainer from '@commonContainers/SkillDropDownContainer/SkillDropDownContainer';
import { resource } from '@assets/text/resources';
import FilenameComponent from '@commonComponents/FileNameComponent/FilenameComponent';
import { ResultFile, Skill, Transaction, TransactionStatus } from '@api/types';
import DownloadDropDownContainer from '@src/common/containers/DownloadDropDownContainer/DownloadDropDownContainer';
import MobileUploadComponent from "@commonComponents/MobileUploadComponent/MobileUploadComponent";
import {createMobileUrl} from "@utils/createMobileUrl";
import {isMobile} from "react-device-detect";
import ReopenQRCodeButtonComponent from "@commonComponents/ReopenQRCodeButtonComponent/ReopenQRCodeButtonComponent";

type CardComponentProps = {
    /** Transaction object */
    transaction: Transaction;
    /** Skill array */
    skills: Array<Skill>;
    /** Event handler for updating skill dropdown */
    handleChange: (transaction: Transaction, skillId: string | null) => void;
    /** Event handler for delete transaction */
    handleDelete: (transaction: Transaction) => void;
    /** Event handler for open QR Code for transaction */
    handleOpen: (transactionId: string) => void;
    /** Event handler for download result of transaction */
    handleDownload: (transaction: Transaction, fileId: string) => void;
};

/**
 *  Component to render transaction card
 */
function CardComponent({ transaction, skills, handleChange, handleDelete, handleDownload, handleOpen }: CardComponentProps) {
    const [mobileURL, setMobileURL] = useState("");
    const handleSkillChange = useCallback(
        (skillId) => {
            handleChange(transaction, skillId);
        },
        [transaction, handleChange],
    );

    const handleDeleteClick = useCallback(() => {
        handleDelete(transaction);
    }, [transaction, handleDelete]);

    const handleOpenQRClick = useCallback(() => {
        if(transaction.transactionId) {
            handleOpen(transaction.transactionId);
        }
    }, [transaction, handleOpen]);

    const handleDownloadClick = (file: ResultFile) => {
        handleDownload(transaction, file.fileId);
    };

    useEffect(()=>{
        if(transaction.transactionId) {
            createMobileUrl(window.location, transaction.transactionId).then(url=>{
                setMobileURL(url);
            })
        }
    },[transaction])


    return (
        <div className={styles.card}>
            <header className={styles.statusSection}>
                <StatusIconComponent status={transaction.status} />
                <StatusTextComponent status={transaction.status} isMobile={transaction.isMobile} />
            </header>

            <section className={styles.previewImageSection}>
                <PreviewComponent transaction={transaction} />
            </section>

            <section className={styles.dropdownSection}>
                <SkillDropdownContainer
                    skills={skills}
                    selectedSkillId={transaction.skillId}
                    handleChange={handleSkillChange}
                    disabled={!!(transaction.isMobile && transaction.transactionId)}
                />
            </section>

            <section className={styles.fileNameSection}>
                {transaction.transactionFile ? (
                    <FilenameComponent filename={transaction.transactionFile.name} />
                ) : transaction.transactionMobileFileName ? (
                    <FilenameComponent filename={transaction.transactionMobileFileName} />
                ) : null}
            </section>

            {transaction.resultClassName && transaction.status === TransactionStatus.PROCESSED ? (
                <section className={styles.resultClassSection}>
                    <span className={styles.resultClassSection__title}>{resource.title.class}: </span>
                    <span title={transaction.resultClassName}>{transaction.resultClassName}</span>
                </section>
            ) : null}

            {transaction.resultFiles ?
                <section className={styles.footerSelectorSection}>
                    <span className={styles.resultSelector}>
                        <DownloadDropDownContainer files={transaction.resultFiles}
                                                   handleDownload={handleDownloadClick}
                                                   isTop
                        />
                    </span>
                    <span />
                    {transaction.deleting ? (
                        <div className={styles.deletingIcon}>
                            <div />
                        </div>
                    ) : (
                        <div className={styles.deleteIcon} onClick={handleDeleteClick}>
                            <div />
                        </div>
                    )}
                </section>:
                <section className={styles.footerSection}>
                    {transaction.reviewLink && transaction.status === TransactionStatus.REVIEW ? (
                        <ReviewResultComponent
                            link={transaction.reviewLink}
                            className={styles.review}
                            text={resource.button.review}
                        />
                    ) : null}
                    {isMobile && transaction.isMobile && mobileURL && transaction.status === TransactionStatus.NEW ? (
                        <MobileUploadComponent
                            text={resource.button.scanDoc}
                            link={mobileURL}
                            className={styles.review}/>
                    ) : null}
                    {!isMobile && transaction.isMobile && mobileURL && transaction.status === TransactionStatus.NEW ? (
                        <ReopenQRCodeButtonComponent
                            text={resource.button.mobileUpload}
                            handleOpen={handleOpenQRClick}
                            className={styles.review}/>
                    ) : null}
                    <span />
                    {transaction.deleting ? (
                        <div className={styles.deletingIcon}>
                            <div />
                        </div>
                    ) : (
                        <div className={styles.deleteIcon} onClick={handleDeleteClick}>
                            <div />
                        </div>
                    )}
                </section>
            }

        </div>
    );
}

export default CardComponent;
