import React, {useCallback, useEffect, useMemo, useState} from 'react';
import {v4 as uuid} from 'uuid';
import {useDispatch, useSelector} from 'react-redux';
import {getSkillsRequest} from '@ducks/Skill/actions/skillActions';
import {createLocalTransactionRequest} from '@ducks/Transaction/actions';
import {errorSelector} from '@ducks/Error/selectors/errorSelector';
import MainPageComponent from './components/MainPageComponent/MainPageComponent';
import {TransactionStatus} from '@api/types';
import {transactionsSelector} from '@ducks/Transaction/selectors/transactionsSelector';
import {useHistory} from 'react-router-dom';
import {getStoredPkce, getToken} from '@utils/tokenUtils';
import {mobileCaptureSelector} from '@ducks/Mobile/selectors/mobileSelector';
import QRCodeModalComponent from '@commonComponents/QRCodeModalComponent/QRCodeModalComponent';
import {getMobileLinkRequest, qrCodeHide} from '@ducks/Mobile/actions/MobileCaptureActions';
import {isMobile} from 'react-device-detect';
import {cardViewSelector} from "@ducks/Common/selectors/commonSelector";
import {flipView} from '@ducks/Common/actions/CommonActions';
import SessionExpiredModalComponent from "@commonComponents/SessionExpiredModalComponent/SessionExpiredModalComponent";

/**
 *  Container to render a Main Page component
 */
function MainPageContainer() {
    const dispatch = useDispatch();
    let history = useHistory();

    const transactions = useSelector(transactionsSelector);
    const mobileCapture = useSelector(mobileCaptureSelector);
    const error = useSelector(errorSelector);
    const isCardView = useSelector(cardViewSelector);
    const authCode = new URLSearchParams(window.location.search).get("code")

    const [showSessionExpired, setShowSessionExpired] = useState<boolean>(false);

    const mobileCaptureTransaction = useMemo(()=> Object.values(transactions.byId)
            .find(transaction => transaction.transactionId == mobileCapture.transactionId)
        , [transactions, mobileCapture]);

    useEffect(() => {
        if (mobileCapture.transactionId && mobileCapture.isVisible && !mobileCapture.mobileInputLink) {
            dispatch(getMobileLinkRequest({transactionId: mobileCapture.transactionId}));
        }
    }, [mobileCapture]);

    useEffect(() => {
        if(isMobile && mobileCapture.mobileInputLink && mobileCapture.isVisible) {
            window.open(mobileCapture.mobileInputLink, '_blank');
        }
    }, [mobileCapture])

    useEffect(() => {
        const checkTokens = async () => {
            const token = await getToken();
            const pkce = await getStoredPkce();
            if (!token && !authCode) {
                setShowSessionExpired(true);
            } else if (authCode) {
                history.push(`/login?code=${authCode}`);
            }
            if(!pkce && !token) {
                history.push(`/login`);
            }
        }
        checkTokens();
    });

    useEffect(() => {
        getToken().then((token) => {
            if (token) {
                dispatch(getSkillsRequest());
            }
        });
    }, []);

    const [drag, setDrag] = useState(false);
    const [, setDragCounter] = useState<number>(0);

    const handleModalClose = useCallback(() => {
        dispatch(qrCodeHide());
    }, [dispatch]);

    const handleSessionModalClose = useCallback(() => {
        setShowSessionExpired(false);
        history.push('/login');
    }, [setShowSessionExpired]);

    const handleSignInModal = useCallback(() => {
        setShowSessionExpired(false);
        history.push('/login');
    }, [setShowSessionExpired]);

    const handleFlip = useCallback(
        () => {
            dispatch(flipView());
        },
        [dispatch],
    );

    const handleDragEnterEvent = useCallback(
        (event: React.DragEvent) => {
            event.preventDefault();
            event.stopPropagation();
            setDragCounter((count: number) => {
                setDrag(true);
                return count + 1;
            });
        },
        [setDragCounter, setDrag],
    );

    const handleDragLeaveEvent = useCallback(
        (event: React.DragEvent) => {
            event.preventDefault();
            event.stopPropagation();
            setDragCounter((count: number) => {
                if (count - 1 <= 0) {
                    setDrag(false);
                }
                return count - 1;
            });
        },
        [setDragCounter, setDrag],
    );

    const handleDragOverEvent = useCallback((event: React.DragEvent) => {
        event.preventDefault();
        event.stopPropagation();
    }, []);

    const handleDropEvent = useCallback(
        (event: React.DragEvent) => {
            event.preventDefault();
            event.stopPropagation();
            setDrag(false);
            if (event.dataTransfer.files && event.dataTransfer.files.length > 0) {
                dispatch(
                    createLocalTransactionRequest({
                        transaction: {
                            uuid: uuid(),
                            status: TransactionStatus.LOCAL,
                            transactionFile: event.dataTransfer.files[0],
                        },
                    }),
                );
                setDragCounter(0);
            }
        },
        [dispatch, setDragCounter, setDrag],
    );

    return (
        <>
            {showSessionExpired?
                <SessionExpiredModalComponent closeHandler={handleSessionModalClose} signInHandler={handleSignInModal}/>
            : null}
            {mobileCapture.isVisible && mobileCaptureTransaction && mobileCaptureTransaction.status==TransactionStatus.NEW && !isMobile ? (
                <QRCodeModalComponent mobileInputLink={mobileCapture.mobileInputLink} closeHandler={handleModalClose} />
            ) : null}
            <MainPageComponent
                transactions={transactions}
                error={error}
                isFlipped={!isCardView}
                handleFlip={handleFlip}
                isDrag={drag}
                handleDragEnter={handleDragEnterEvent}
                handleDragLeave={handleDragLeaveEvent}
                handleDragOver={handleDragOverEvent}
                handleDrop={handleDropEvent}
            />
        </>
    );
}

export default MainPageContainer;
