import React from 'react';
import ReactCardFlip from 'react-card-flip';
import { isMobile } from 'react-device-detect';
import MainPageFrontComponent from '../MainPageFrontComponent/MainPageFrontComponent';
import MainPageBackComponent from '../MainPageBackComponent/MainPageBackComponent';

import UploadIcon from '@assets/icons/ic_upload.svg';
import styles from './MainPageComponent.module.scss';
import ErrorIcon from '@assets/icons/ic_error.svg';
import { TransactionState } from '@ducks/Transaction/reducer';
import { ErrorState } from '@ducks/Error/reducer';
import { resource } from '@assets/text/resources';
import MainPageMobileComponent from '@routes/MainPage/components/MainPageMobileComponent/MainPageMobileComponent';
import MobileCardViewPageComponent from '@routes/CardViewPage/MobileCardViewPageComponent';
import ViewPageComponent from "@routes/ViewPage/ViewPageComponent";

interface MainPageComponentProperties {
    /** Transactions (normalized object) */
    transactions: TransactionState;
    /** Error */
    error: ErrorState;
    /** Flip card flag */
    isFlipped: boolean;
    /** Handler for flip event */
    handleFlip: (event: React.MouseEvent) => void;
    /** Drag file flag */
    isDrag: boolean;
    /** Handler for drag enter event */
    handleDragEnter: (event: React.DragEvent) => void;
    /** Handler for drag leave event */
    handleDragLeave: (event: React.DragEvent) => void;
    /** Handler for drag over event */
    handleDragOver: (event: React.DragEvent) => void;
    /** Handler for drop event */
    handleDrop: (event: React.DragEvent) => void;
}

/**
 *  Component to render a Main Page with flip-flop
 */
function MainPageComponent({
    transactions,
    error,
    isFlipped,
    handleFlip,
    isDrag,
    handleDragEnter,
    handleDragLeave,
    handleDragOver,
    handleDrop,
}: MainPageComponentProperties) {
    const errorStyle = error.isVisible ? styles.open : styles.close;
    const cardParameters = { front: { zIndex: '0' , transformStyle: null}, back: {transformStyle: null}};
    if (isMobile) {
        return (
            <>
                {transactions && transactions.allIds.length > 0 ? (
                    <MobileCardViewPageComponent />
                ) : (
                    <MainPageMobileComponent />
                )}
            </>
        );
    }
    return (
        <div
            className={styles.container}
            onDragEnter={handleDragEnter}
            onDragOver={handleDragOver}
            onDragLeave={handleDragLeave}
            onDrop={handleDrop}
        >
            {isDrag ? (
                <div className={styles.dropzone}>
                    <UploadIcon />
                    <p className={styles.droptext}>{resource.title.dropDocToUpload}</p>
                </div>
            ) : undefined}
            <div className={`${styles.notification} ${errorStyle}`}>
                <ErrorIcon  className={styles.errorIcon}/>
                {error.text}
            </div>
            {transactions && transactions.allIds.length > 0 ? (
                <ViewPageComponent isCardView={!isFlipped}/>
            ) : (
                <ReactCardFlip
                    containerStyle={{ height: '100%' }}
                    cardStyles={cardParameters}
                    isFlipped={isFlipped}
                    flipDirection="horizontal"
                    infinite
                >
                    <MainPageFrontComponent handleFlip={handleFlip} />
                    <MainPageBackComponent  handleFlip={handleFlip} />
                </ReactCardFlip>
            )}
        </div>
    );
}

export default MainPageComponent;
