import React, {useCallback, useEffect, useState} from 'react';
import styles from './TableRowComponent.module.scss';
import StatusTextComponent from '@commonComponents/StatusTextComponent/StatusTextComponent';
import StatusIconComponent from '@commonComponents/StatusIconComponent/StatusIconComponent';
import FilenameComponent from '@commonComponents/FileNameComponent/FilenameComponent';
import ReviewResultComponent from '@commonComponents/ReviewResultComponent/ReviewResultComponent';

import DeleteIcon from '@assets/icons/ic_delete.svg';
import DeletingIcon from '@assets/icons/ic_deleting.svg';
import SkillDropdownContainer from '@commonContainers/SkillDropDownContainer/SkillDropDownContainer';
import { resource } from '@assets/text/resources';
import {ResultFile, Skill, Transaction, TransactionStatus} from '@api/types';
import DownloadDropDownContainer from "@commonContainers/DownloadDropDownContainer/DownloadDropDownContainer";
import {isMobile} from "react-device-detect";
import ReopenQRCodeButtonComponent from "@commonComponents/ReopenQRCodeButtonComponent/ReopenQRCodeButtonComponent";
import {createMobileUrl} from "@utils/createMobileUrl";

interface ITableRowComponentProperties {
    /**  Transaction object */
    transaction: Transaction;
    /** Array of skills */
    skills: Array<Skill>;
    /** Event handler for updating skill dropdown */
    handleChange: (transaction: Transaction, skillId: string | null) => void;
    /** Event handler for delete transaction */
    handleDelete: (transaction: Transaction) => void;
    /** Event handler for download result of transaction */
    handleDownload: (transaction: Transaction, fileType: string) => void;
    /** Event handler for open QR Code for transaction */
    handleOpen: (transactionId: string) => void;
}

/**
 *  Component to render transaction table row
 */
function TableRowComponent({
    transaction,
    skills,
    handleChange,
    handleDelete,
    handleDownload,
    handleOpen
}: ITableRowComponentProperties) {
    const [mobileURL, setMobileURL] = useState("");
    const handleSkillChange = useCallback(
        (skillId) => {
            handleChange(transaction, skillId);
        },
        [handleChange, transaction],
    );

    const handleOpenQRClick = useCallback(() => {
        if(transaction.transactionId) {
            handleOpen(transaction.transactionId);
        }
    }, [transaction, handleOpen]);

    const handleDeleteClick = useCallback(() => {
        handleDelete(transaction);
    }, [handleDelete, transaction]);

    const handleDownloadClick = (file: ResultFile) => {
        handleDownload(transaction, file.fileId);
    };

    useEffect(()=>{
        if(transaction.transactionId) {
            createMobileUrl(window.location, transaction.transactionId).then(url=>{
                setMobileURL(url);
            })
        }
    },[transaction])

    return (
        <div className={styles.container}>
            <span>
                <StatusIconComponent status={transaction.status} />
            </span>
            <span>
                <SkillDropdownContainer
                    skills={skills}
                    selectedSkillId={transaction.skillId}
                    handleChange={handleSkillChange}
                />
            </span>
            <span>
                {transaction.transactionFile ? (
                    <FilenameComponent filename={transaction.transactionFile.name} />
                ) : transaction.transactionMobileFileName ? (
                    <FilenameComponent filename={transaction.transactionMobileFileName} />
                ) : null}
            </span>
            <span>
                <StatusTextComponent status={transaction.status} isMobile={transaction.isMobile}/>
            </span>
            <span />
            <span id="Result">
                {transaction.resultClassName && transaction.status === TransactionStatus.PROCESSED ? (
                    <span className={styles.resultClass}>
                        <span className={styles.resultClass__title}>{resource.title.class}: </span>
                        <span title={transaction.resultClassName}>{transaction.resultClassName}</span>
                    </span>
                ) : null}
                {transaction.reviewLink && transaction.status === TransactionStatus.REVIEW ? (
                    <ReviewResultComponent
                        text={resource.button.reviewResult}
                        link={transaction.reviewLink}
                        className={styles.review}
                    />
                ) : null}
                {!isMobile && transaction.isMobile && mobileURL && transaction.status === TransactionStatus.NEW ? (
                    <ReopenQRCodeButtonComponent
                        text={resource.button.mobileUpload}
                        handleOpen={handleOpenQRClick}
                        className={styles.review}/>
                ) : null}
                {transaction.resultFiles && transaction.resultFiles.length>0 && transaction.status === TransactionStatus.PROCESSED ? (
                    <span className={styles.resultSelector}>
                        <DownloadDropDownContainer files={transaction.resultFiles}
                                                   handleDownload={handleDownloadClick}/>
                    </span>
                ) : null}
            </span>
            <span id="Delete">
                {transaction.deleting ? (
                    <DeletingIcon className={styles.deletingIcon} />
                ) : (
                    <DeleteIcon className={styles.deleteIcon} onClick={handleDeleteClick} />
                )}
            </span>
        </div>
    );
}

export default TableRowComponent;
